@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use v5.10;
 
-our $VERSION = '0.17';
+our $VERSION = '0.18';
 
 use RDF::NS;
 use Carp qw(croak carp);
@@ -30,6 +30,7 @@ use constant datatypeString => qr/^(.*?)[\^]
                                   ((($PREFIX)?_($NAME))|<([a-z][a-z0-9+.-]*:.*)>)$/x;
 
 use constant explicitIRIlike => qr/^<(.+)>$/;
+use constant xsd_string => 'http://www.w3.org/2001/XMLSchema#string';
 
 sub new {
     my ($class, %options) = @_;
@@ -102,23 +103,26 @@ sub decode {
         # predicate map    
 
         my $id = $map->{_id};
-        if ($self->is_null($id,'_id')) {
-            return;
-        }
+        return if $self->is_null($id,'_id');
 
         my $subject = $id ne '' ? $self->expect_resource($id,'subject') : undef;
         if (defined $subject and $subject ne '') {
             $self->predicate_map( $subject, $map );
         } elsif ($self->{strict}) { 
-            $self->error("invalid subject: ".(ref $id ? reftype($id) : $id));
+            $self->error("invalid subject", $id);
         }
 
     } else {
-        # subject map
-        
-        for my $key (grep { $_ ne '_ns' } keys %$map) {
-            next if $key eq '' and !$self->{strict};
-            my $subject = $self->expect_resource($key,'subject') // next;
+        # 3.4.1 subject maps
+        foreach my $key (grep { $_ ne '_' and $_ !~ /^_[^:]/ } keys %$map) {
+            next if $self->is_null($key,'subject');
+
+            my $subject = $self->subject($key);
+            if (!$subject) {
+                $self->error("invalid subject", $key);
+                next;
+            }
+
             my $predicates = $map->{$key};
             if (exists $predicates->{_id} and ($self->resource($predicates->{_id}) // '') ne $subject) {
                 $self->error("subject _id must be <$subject>");
@@ -139,7 +143,7 @@ sub predicate_map {
 
         my $predicate = do {
             if ($_ eq 'a') {
-                "http://www.w3.org/1999/02/22-rdf-syntax-ns#type";
+                'http://www.w3.org/1999/02/22-rdf-syntax-ns#type';
             } elsif ( $_ =~ /^<(.+)>$/ ) {
                 $self->iri($1);
             } elsif ( $_ =~ qName ) { 
@@ -210,9 +214,9 @@ sub expect_resource {
 sub resource {
     my ($self, $r) = @_;
     
-    if (!defined $r) {
-        undef;
-    } elsif ( $r =~ explicitIRIlike ) {
+    return unless defined $r;
+
+    if ( $r =~ explicitIRIlike ) {
         $self->iri($1);
     } elsif ( $r =~ blankNode ) {
         $self->blank_identifier($1);
@@ -221,7 +225,23 @@ sub resource {
     } elsif ( $r =~ IRIlike )  {
         $self->iri($r);
     } else {
-        undef;
+        undef
+    }
+}
+
+sub subject { # plain IRI, qName, or blank node
+    my ($self, $s) = @_;
+
+    return unless defined $s;
+
+    if ( $s =~ IRIlike )  {
+       $self->iri($s);
+    } elsif ( $s =~ qName ) {
+       $self->prefixed_name($1,$2);
+    } elsif ( $s =~ blankNode ) {
+       $self->blank_identifier($1);
+    } else {
+        undef
     }
 }
 
@@ -243,14 +263,14 @@ sub object {
     } elsif ( $o =~ datatypeString ) {
         if ($6) {
             my $datatype = $self->iri($6) // return;
-            if ($datatype eq 'http://www.w3.org/2001/XMLSchema#string') {
+            if ($datatype eq xsd_string) {
                 [$1,undef];
             } else {
                 [$1,undef,$datatype];
             }
         } else {
             my $datatype = $self->prefixed_name($4,$5) // return;
-            if ($datatype eq 'http://www.w3.org/2001/XMLSchema#string') {
+            if ($datatype eq xsd_string) {
                 [$1,undef];
             } else {
                 [$1,undef,$datatype];
@@ -299,9 +319,13 @@ sub triple {
 }
 
 sub error {
-    my ($self, $message, $node) = @_;
+    my ($self, $message, $value, $context) = @_;
 
-    # TODO: include $node instead of $message (bless $message, 'RDF::aREF::Error')
+    # TODO: include $context (bless $message, 'RDF::aREF::Error')
+
+    if (defined $value) {
+        $message .= ': ' . (ref $value ? reftype $value : $value);
+    }
     
     if (!$self->{complain}) {
         return;
